<?php
/*
 * Copyright(c) 2015 PAYDESIGN, Inc. All rights reserved.
 * http://www.paydesign.co.jp/
 */

namespace Plugin\PayDesign\Controller;

use Eccube\Application;
use Plugin\PayDesign\Controller\Util\PluginUtil;
use Plugin\PayDesign\Form\Type\ConvenienceType;
use Plugin\PayDesign\Service\client\PAYDESIGN_PLG_DCHK_CREDIT;
use Plugin\PayDesign\Service\client\PAYDESIGN_PLG_DCHK_PERIODIC;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * Controller to handle module setting screen
 */
class CreditController {

    public $app;
    public $const;

    // 【フロントページ】クレジット決済：通知ページ
    public function result(\Eccube\Application $app) {

        $this->app = $app;
        $this->const = $app['config']['PayDesign']['const'];
        $rcode = '0' ;
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();

        $objMdl->printLog("クレジット決済：入金通知処理開始");

        if(count($_REQUEST) == 0) {
            $objMdl->printLog("クレジット決済：受信データ存在せず 入金通知処理終了");
            $rcode = '9';
        }

        if( $rcode == '0' ) {
            // パラメータ構築
            $arrReq = array();
            $request_msg = "クレジット決済：入金通知処理 受信パラメータ\n" ;
            $i = 1;
            foreach($_REQUEST as $key=>$val){
                $arrReq[$key] = $val;
                $request_msg .= "　".$i.":".$key." ... ".$val."\n";
                $i++;
            }
            $objMdl->printLog($request_msg);

            if( strstr($_REQUEST["SID"], $this->const['PDP_CARD_PREFIX_NORMAL'])
             || strstr($_REQUEST["SID"], $this->const['PDP_CARD_PREFIX_PERIODIC']) ) {
                // 会員カードデータ更新
                $objMdl->printLog("クレジット決済：カード更新レスポンス（SID:".$_REQUEST["SID"]."） 入金通知処理終了");
                $rcode = $this->updatePaydesginCardInfoByMypage();
            } else {
                if(isset($_REQUEST["TIME"])){
                    //完了通知
                    $arrCvs = $objMdl->getCreditInfo();
                    if(in_array($_REQUEST["CVS"],$arrCvs)){
                        // 2017.1 SIDから注文IDを抽出
                        $order_id = NULL;
                        $userSettings = $objMdl->getUserSettings();
                        $sid_len = mb_strlen($_REQUEST["SID"]);
                        $prefix_word = $this->const['PDP_CARD_PREFIX_UID'].$userSettings['prefix_word'];
                        // prefix_wordが存在しない場合はリンク式決済なので、prefix_word作り直し
                        if( !strstr($_REQUEST["SID"], $prefix_word) ) { $prefix_word = $userSettings['prefix_word']; }
                        $prefix_word_len = mb_strlen($prefix_word);
                        $order_id = mb_substr($_REQUEST["SID"], $prefix_word_len, $sid_len-$prefix_word_len);
                        $objMdl->printLog("SIDから取得した注文ID:".$order_id);

                        // 受注拡張データ更新
                        $rcode = $this->updatePaydesignOrderInfo($order_id);
                        // 会員カードデータ更新
                        if( $rcode == '0' ) { $rcode = $this->updatePaydesginCardInfo($order_id); }
                        // 受注データ更新
                        if( $rcode == '0' ) { $rcode = $this->updateOrder($order_id); }
                    }else{
                        $objMdl->printLog("クレジット決済：不正データ（クレジットカード情報存在せず） 入金通知処理終了");
                        $rcode = '9';
                    }
                }else{
                    $objMdl->printLog("クレジット決済：不正通知パラメータ（完了通知ではない） 入金通知処理終了");
                    $rcode = '9';
                }
            }
        }

        // セッション情報が残っている場合は削除
        /*
        if( !is_null($this->app['session']->get('eccube.plugin.paydesign.orderId')) ) {
            $this->app['session']->remove('eccube.plugin.paydesign.orderId'); }
        */

        // 終了
        $objMdl->printLog("クレジット決済：入金通知処理終了[".$rcode."]");
        if( $objMdl->checkEccubeOldVersion011() ) {
            // 3.0.10以前のバージョン
            header("Content-Type: text/plain; charset=Shift_JIS");
            echo $rcode;
            exit;
        } else {
            // 3.0.11以降のバージョン
            $response = new Response(
                'Content',
                Response::HTTP_OK,
                array( 'Content-Type' => 'text/plain')
            );
            $response->setCharset('Shift_JIS');
            $response->setContent($rcode);
            $response->prepare($this->app['request']);
            $response->send();
            return $response;
        }
    }

    // 【フロントページ】リンク型クレジット決済：完了ページ（外部リンクからの決済実行処理）
    public function linkResult(\Eccube\Application $app)
    {
        $this->app = $app;
        $this->const = $app['config']['PayDesign']['const'];
        $error_message = "" ;

        $objMdl =& PluginUtil::getInstance($app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // 注文情報の取得
        if( !is_null($this->app['eccube.service.cart']->getPreOrderId()) ) {
            $Order = $this->app['eccube.repository.order']->findOneBy(array('pre_order_id' => $this->app['eccube.service.cart']->getPreOrderId()));
        } else {
            // pre_order_id が無い場合、セッション情報に保存したorder_idからの取得を試みる
            $session = $this->app['request']->getSession();
            if( !is_null($session->get('eccube.plugin.paydesign.orderId')) ) {
                $Order = $this->app['eccube.repository.order']->find($session->get('eccube.plugin.paydesign.orderId'));
            }
        }

        // 注文情報取得チェック
        if (is_null($Order)) {
            $error_message = "注文情報の取得が出来ませんでした。この手続きは無効となりました。";
        } else {
            $order_id = $Order->getId();
        }

        // 通信METHODチェック
        if( $error_message == "" && $_SERVER['REQUEST_METHOD'] === 'GET' ) {
            // GETパラメータチェック
            $check_sid = $userSettings['prefix_word'].$order_id;
            if($_GET['SID'] === $check_sid) {
                // 正常処理（注文完了）
                // カート初期化(削除)
                $cartService = $this->app['eccube.service.cart'];
                $cartService->clear()->save();
                // ログ
                $msg = "リンク型クレジット決済 注文完了処理（SID=".$check_sid."）";
                $objMdl->printLog($msg);
                // メール送信
                $this->app['eccube.service.mail']->sendOrderMail($Order);
                $this->app['session']->set('eccube.plugin.paydesign.orderId', $order_id);
                // 注文情報の終了処理
                $objClientCredit = new PAYDESIGN_PLG_DCHK_CREDIT($this->app);
                $objClientCredit->changeOrderData($Order);
            } else {
                $error_message = "SID(GET:".$_GET['SID'].", PARAM:".$check_sid.")が一致しません。この手続きは無効となりました。";
            }
        } else {
            $error_message = '不正な'.$_SERVER['REQUEST_METHOD'].'リクエストを受信しました';
        }

        if($error_message != "") {
            // エラー発生時はエラーページの表示
            return $this->app['view']->render('error.twig', array('error_message' => $error_message));
        } else {
            // 正常終了時は注文完了ページに遷移
            return $this->app->redirect($this->app['url_generator']->generate('shopping_complete'));
        }
    }

    // 【フロントページ】リンク型クレジット決済：キャンセルページ（外部リンクからの戻る処理）
    public function cancel(\Eccube\Application $app)
    {
        $this->app = $app;
        $this->const = $app['config']['PayDesign']['const'];

        // 受注情報を取得
        if( !is_null($this->app['eccube.service.cart']->getPreOrderId()) ) {
            $Order = $this->app['eccube.repository.order']->findOneBy(array('pre_order_id' => $this->app['eccube.service.cart']->getPreOrderId()));
        } else {
            // pre_order_id が無い場合、セッション情報に保存したorder_idからの取得を試みる
            $session = $this->app['request']->getSession();
            if( !is_null($session->get('eccube.plugin.paydesign.orderId')) ) {
                $Order = $this->app['eccube.repository.order']->find($session->get('eccube.plugin.paydesign.orderId'));
            }
        }

        // 受注情報を更新（購入処理中として更新する）
        if( !is_null($Order) ) {
            $Order->setOrderStatus($app['eccube.repository.order_status']->find($app['config']['order_processing']));
            $this->app['orm.em']->persist($Order);
            $this->app['orm.em']->flush();
        }

        // セッション情報が残っている場合は削除
        if( !is_null($this->app['session']->get('eccube.plugin.paydesign.orderId')) ) {
            $this->app['session']->remove('eccube.plugin.paydesign.orderId'); }

        return $this->app->redirect($this->app->url('shopping'));
    }

    // 【フロントページ】UID型クレジットカード注文完了ページ（メール送信を行う）
    public function creditComplete(\Eccube\Application $app)
    {
        // 初期化
        $this->app = $app;
        $this->const = $app['config']['PayDesign']['const'];
        $session = $this->app['request']->getSession();

        if( !is_null($session->get('eccube.plugin.paydesign.orderId')) ) {
            // 注文データの取得
            $Order = $this->app['eccube.repository.order']->find($session->get('eccube.plugin.paydesign.orderId'));
            // メール送信
            $this->app['eccube.service.mail']->sendOrderMail($Order);
        }

        // 完了ページに遷移
        return $this->app->redirect($this->app->url('shopping_complete'));
    }

    // 【フロントページ】Mypage リンク型クレジットカード情報更新：完了ページ（外部リンクからの決済実行処理）
    public function mypage_link_card_update_result(\Eccube\Application $app) {

        // 初期化
        $this->app = $app;
        $this->const = $app['config']['PayDesign']['const'];
        $error_message = "" ;
        $objMdl =& PluginUtil::getInstance($app);

        // セッション情報（クレジットカード登録用注文IDの削除）
        $session = $this->app['request']->getSession();
        $card_order_id = $session->get('eccube.mypage.paydesign.card_order_id');

        // クレジットカード更新用注文情報を取得
        $PayDesignCardOrder = $this->app['eccube.plugin.paydesign.repository.paydesign_card_order']->find($card_order_id);
        // 注文情報取得チェック
        if (is_null($PayDesignCardOrder)) {
            $error_message = "クレジットカード更新用注文情報の取得が出来ませんでした。この手続きは無効となりました。";
        }

        // 通信METHODチェック
        if( $error_message == "" && $_SERVER['REQUEST_METHOD'] === 'GET' ) {
            // GETパラメータチェック
            $objMdl->install();
            $check_sid = $PayDesignCardOrder->getCardOrderSid();
            if($_GET['SID'] === $check_sid) {
                // 正常処理（注文完了）
                // ログ
                $msg = "クレジットカード更新用注文完了処理（SID=".$check_sid."）";
                $objMdl->printLog($msg);
                // メール送信用データ作成
                $BaseInfo = $this->app['eccube.repository.base_info']->get();
                $subject = '[' . $BaseInfo->getShopName() . '] ' . $this->const['PDP_CREDIT_UPDATE_SUBJECT'];
                $Customer = $this->app['eccube.repository.customer']->find($PayDesignCardOrder->getCustomerId());
                $to_mailaddr = $Customer->getEmail();
                $card_kind = '通常注文用';
                if( $PayDesignCardOrder->getCardKind() == $this->const['PDP_CARD_KIND_PERIODIC'] ) { $card_kind = '定期注文用'; }
                $formData = array(
                    'name01'    => $Customer->getName01(),
                    'name02'    => $Customer->getName02(),
                    'shop_name' => $BaseInfo->getShopName(),
                    'card_kind' => $card_kind,
                    'sid'       => $PayDesignCardOrder->getCardOrderSid(),
                    'email'     => $BaseInfo->getEmail01(),
                );
                $body = $this->app->renderView("PayDesign/View/mail_template/update_credit.twig", array(
                    'data' => $formData,
                ));
                // メール送信
                $objMdl->sendMail( $subject, $to_mailaddr, $body );
                // クレジットカード更新用注文情報とカード情報の更新
                $objClientCredit = new PAYDESIGN_PLG_DCHK_CREDIT($this->app);
                $objClientCredit->changeCardOrderData($PayDesignCardOrder);
            } else {
                $error_message = "SID(GET:".$_GET['SID'].", PARAM:".$check_sid.")が一致しません。この手続きは無効となりました。";
            }
        } else {
            $error_message = '不正な'.$_SERVER['REQUEST_METHOD'].'リクエストを受信しました';
        }

        if($error_message != "") {
            // エラー発生時はエラーページの表示
            return $this->app['view']->render('error.twig', array('error_message' => $error_message));
        } else {
            // 正常終了時はセッション情報の削除
            $session->remove('eccube.mypage.paydesign.card_order_id');

            // 正常終了時は完了ページに遷移
            return $this->app->redirect($this->app['url_generator']->generate('paydesign_mypage_credit_complete'));
        }
    }

    // 【フロントページ】Mypage リンク型クレジットカード情報更新：キャンセルページ（外部リンクからの戻る処理）
    public function mypage_link_card_update_cancel(\Eccube\Application $app) {

        // 初期化
        $this->app = $app;
        $this->const = $app['config']['PayDesign']['const'];

        // クレジットカード登録用注文情報を取得
        $session = $this->app['request']->getSession();
        $card_order_id = $session->get('eccube.mypage.paydesign.card_order_id');
        if( !is_null($card_order_id) && isset($card_order_id) ) {
            $PayDesignCardOrder = $this->app['eccube.plugin.paydesign.repository.paydesign_card_order']->find($card_order_id);
            if( !is_null($PayDesignCardOrder) && isset($PayDesignCardOrder) ) {
                // 受注情報を更新（新規に戻す（ステータスを更新））
                $PayDesignCardOrder->setCardOrderStatus($this->const['PDP_CARD_ORDER_STATUS_NEW']);
                $this->app['orm.em']->persist($PayDesignCardOrder);
                $this->app['orm.em']->flush();
            }
        }

        // マイページに戻る
        return $this->app->redirect($this->app->url('paydesign_mypage_credit'));
    }

    // 受注拡張データ更新
    private function updatePaydesignOrderInfo($order_id = NULL) {

        // 戻り値初期化
        $rcode = '0' ;

        // プラグインユーティリティオブジェクトの作成
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // 現在の受注拡張データ取得
        $objMdl->printLog("受注拡張データ更新：受注拡張データ取得開始");
        $PaydesignOrderInfo =
            $this->app['eccube.plugin.paydesign.repository.paydesign_order_info']->findOneBy( array( 'sid' => $_REQUEST["SID"]) );

        if( is_null($PaydesignOrderInfo) ) {
            $objMdl->printLog("受注拡張データ更新：受注拡張データ取得・更新失敗（1回目）");
            if( !is_null($order_id) ) {
                // 受注拡張データ新規作成 
                $PaydesignOrderInfo = new \Plugin\PayDesign\Entity\PayDesignOrderInfo() ;
                $objMdl->printLog("受注拡張データ更新：受注拡張データ新規作成");
                $PaydesignOrderInfo
                    ->setOrderId($order_id)
                    ->setSid($_REQUEST["SID"])
                    ->setOrderKind($this->app['config']['PayDesign']['const']['PDP_ORDER_KIND_NORMAL'])
                    ->setCreditUidCardSelectFlg(1)
                    ->setCheckCount(0);
            } else {
                $objMdl->printLog("受注拡張データ更新：受注ID取得失敗");
                $rcode = '9';
            }
        }

        if( !is_null($PaydesignOrderInfo) ) {
            // データの設定・更新
            $objMdl->printLog("受注拡張データ更新：受注拡張データ更新開始");
            $now = new \DateTime();
            $PaydesignOrderInfo
                ->setResultSeq($_REQUEST["SEQ"])
                ->setResultDate($_REQUEST["DATE"])
                ->setResultTime($_REQUEST["TIME"])
                ->setResultIp($_REQUEST["IP"])
                ->setResultKingaku($_REQUEST["KINGAKU"])
                ->setResultCvs($_REQUEST["CVS"])
                ->setResultScode($_REQUEST["SCODE"])
                ->setResultShonin($_REQUEST["SHONIN"])
                ->setResultHishimuke($_REQUEST["HISHIMUKE"])
                ->setResultFuka($_REQUEST["FUKA"])
                ->setResultPaymode($_REQUEST["PAYMODE"])
                ->setResultIncount($_REQUEST["INCOUNT"])
                ->setResultIpUserId($_REQUEST["IP_USER_ID"])
                ->setResultReceiveDate($now);
            $this->app['orm.em']->persist($PaydesignOrderInfo);
            $this->app['orm.em']->flush();
        } else {
            // データが取得できなかった場合はエラーログ出力
            $objMdl->printLog("受注拡張データ更新：受注拡張データ取得・更新失敗（2回目）");
            $rcode = '9';
        }

        // 不要な受注拡張データがある場合は削除する
        if( $rcode != '9' ) {
            $check_sid = "";
            $uid_prefix_word = $this->const['PDP_CARD_PREFIX_UID'].$userSettings['prefix_word'];
            if( strstr($_REQUEST["SID"], $uid_prefix_word) ) {
                // 取得SIDがUID用のPREFIXありだったので、チェック対象は UID用のPREFIXなし
                $check_sid = $userSettings['prefix_word'].$order_id;
            } else {
                // 取得SIDがUID用のPREFIXなしだったので、チェック対象は UID用のPREFIXあり
                $check_sid = $uid_prefix_word.$order_id;
            }
            $objMdl->printLog("受注拡張データ更新：受注拡張データ削除チェック用SID：".$check_sid);
            $DeletePaydesignOrderInfo =
                $this->app['eccube.plugin.paydesign.repository.paydesign_order_info']->findOneBy( array( 'sid' => $check_sid ) );
            // 削除用データが取得出来た場合は削除処理を実行
            if( !is_null($DeletePaydesignOrderInfo) ) {
                $objMdl->printLog("受注拡張データ更新：受注拡張データ削除が存在するので実行");
                $this->app['orm.em']->remove($DeletePaydesignOrderInfo);
                $this->app['orm.em']->flush();
            }
        }

        $objMdl->printLog("受注拡張データ更新：受注拡張データ取得・更新終了");

        return $rcode;
    }

    // 会員カードデータ更新
    private function updatePaydesginCardInfo($order_id = NULL) {

        // 戻り値初期化
        $rcode = '0' ;
        $insert_flg = false;

        // プラグインユーティリティオブジェクトの作成
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        $objMdl->printLog("会員カードデータ更新：会員カードデータ取得・更新開始");

        // 現在の受注拡張データ取得
        $PaydesignOrderInfo =
            $this->app['eccube.plugin.paydesign.repository.paydesign_order_info']->findOneBy( array( 'sid' => $_REQUEST["SID"]) );

        // 対象の受注データ取得(会員IDの取得が目的)
        $Order = NULL;
        if( is_null( $PaydesignOrderInfo ) ) {
            $objMdl->printLog("会員カードデータ更新：受注拡張データ取得失敗");
            if( !is_null( $order_id ) ) { $Order = $this->app['eccube.repository.order']->find($order_id); }
        } else {
            $objMdl->printLog("会員カードデータ更新：受注拡張データ取得成功");
            $Order = $this->app['eccube.repository.order']->find($PaydesignOrderInfo->getOrderId());
        }
        // 受注データが取得できない場合は終了
        if( is_null($Order) ) {
            $objMdl->printLog("会員カードデータ更新：受注データ取得失敗"); $rcode = '9'; return $rcode; }

        if( is_null( $Order->getCustomer() ) ) {
            // 非会員の場合は正常処理終了とする（会員カードデータは存在しない）
            $objMdl->printLog("会員カードデータ更新：会員カードデータ取得・更新終了（非会員のため処理不要）");
            return $rcode;
        } else {
            // 会員IDの取得
            $customer_id = $Order->getCustomer()->getId();
        }

        // IP_USER_IDが無い場合は、以降の処理を行わないが、処理不要として正常終了
        if( is_null( $_REQUEST["IP_USER_ID"]) || '' === $_REQUEST["IP_USER_ID"] ) {
            $objMdl->printLog("会員カードデータ更新：会員カードデータ取得・更新終了（IP_USER_ID無しのため処理不要）");
            return $rcode;
        }

        // 対象カード種別を取得
        $card_kind = $this->const['PDP_CARD_KIND_NORMAL'];
        if( strstr( $_REQUEST["SID"], $this->const['PDP_CARD_PREFIX_PERIODIC']) ) { $card_kind = $this->const['PDP_CARD_KIND_PERIODIC']; }

        // 現在の会員カードデータ取得
        $PaydesignCardInfo =
            $this->app['eccube.plugin.paydesign.repository.paydesign_card_info']->findOneBy(
                array( 'customer_id' => $customer_id,
                       'card_sid'    => $_REQUEST["IP_USER_ID"]
            ));
        if( is_null($PaydesignCardInfo) ) {
            $objMdl->printLog("会員カードデータ更新：会員カードデータ取得失敗");
            $rcode = '9';
            return $rcode;
        }

        // データの設定（更新時情報）
        $now = new \DateTime();
        $card_num   = substr($_REQUEST["SCODE"], 0, 4); // カード番号（下4桁）
        $card_limit = substr($_REQUEST["SCODE"], 4, 4); // カード有効期限
        $PaydesignCardInfo
            ->setCardNum($card_num)
            ->setCardLimit($card_limit)
            ->setCardEnable(1)
            ->setCardOrderSid($_REQUEST["SID"])
            ->setUpdateDate($now);

        // データの更新
        $this->app['orm.em']->persist($PaydesignCardInfo);
        $this->app['orm.em']->flush();

        $objMdl->printLog("会員カードデータ更新：会員カードデータ取得・更新終了");

        return $rcode;
    }

    // 会員データ更新（Myページからのカード情報更新時）
    private function updatePaydesginCardInfoByMypage() {

        $rcode = '0';

        // プラグインユーティリティオブジェクトの作成
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // Start
        $objMdl->printLog("Myページからの会員カードデータ更新：会員カードデータ取得・更新開始");

        // 現在のクレジットカード更新用注文情報を取得
        $PayDesignCardOrder =
            $this->app['eccube.plugin.paydesign.repository.paydesign_card_order']->findOneBy(
                array( 'card_order_sid' => $_REQUEST["SID"],
            ));
        if( is_null($PayDesignCardOrder) ) {
            // データ取得失敗
            $objMdl->printLog("Myページからの会員カードデータ更新：現在のクレジットカード更新用注文情報を取得失敗");
            $rcode = '9';
            return $rcode;
        }

        // 現在の会員カードデータ取得
        $PayDesignCardInfo =
            $this->app['eccube.plugin.paydesign.repository.paydesign_card_info']->findOneBy(
                array( 'card_sid' => $PayDesignCardOrder->getCardIpUserId(),
            ));
        if( is_null($PayDesignCardInfo) ) {
            // データ取得失敗
            $objMdl->printLog("Myページからの会員カードデータ更新：現在の会員カード情報を取得失敗");
            $rcode = '9';
            return $rcode;
        }

        // データの更新
        $now = new \DateTime();
        $card_num   = substr($_REQUEST["SCODE"], 0, 4); // カード番号（下4桁）
        $card_limit = substr($_REQUEST["SCODE"], 4, 4); // カード有効期限
        $PayDesignCardInfo
            ->setCardNum($card_num)
            ->setCardLimit($card_limit)
            ->setCardEnable(1)
            ->setUpdateDate($now);

        // データの登録・更新
        $this->app['orm.em']->persist($PayDesignCardInfo);
        $this->app['orm.em']->flush();

        // End
        $objMdl->printLog("Myページからの会員カードデータ更新：会員カードデータ取得・更新終了");

        return $rcode;
    }

    // 受注データ更新
    private function updateOrder($order_id = NULL) {

        // 戻り値初期化
        $rcode = '0' ;
        $now = new \DateTime();

        // プラグインユーティリティオブジェクトの作成
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        $objMdl->printLog("受注データ更新：受注データ取得・更新開始");

        // クレジットカード更新処理の場合は処理不要のため終了
        if( strstr( $_REQUEST["SID"], $this->const['PDP_CARD_PREFIX_NORMAL'])
         || strstr( $_REQUEST["SID"], $this->const['PDP_CARD_PREFIX_PERIODIC']) ) {
            $objMdl->printLog("受注データ更新：受注データ取得・更新終了(クレジットカード更新処理)");
            return $rcode;
        }

        // 現在の受注拡張データ取得
        $PaydesignOrderInfo =
            $this->app['eccube.plugin.paydesign.repository.paydesign_order_info']->findOneBy( array( 'sid' => $_REQUEST["SID"]) );

        $Order = NULL;
        if( is_null($PaydesignOrderInfo) ) {
            $objMdl->printLog("受注データ更新：受注拡張データ取得失敗");

            // 対象の受注データ取得(会員IDの取得が目的)
            if( !is_null( $order_id ) ) { $Order = $this->app['eccube.repository.order']->find($order_id); }
        } else {
            $objMdl->printLog("受注データ更新：受注拡張データ取得成功");

            // 受注拡張データが取得出来ている場合はFUKA情報の取得
            $fuka = "";
            $fuka = $PaydesignOrderInfo->getResultFuka();
            if( $fuka != "" ) { $objMdl->printLog("取得したFUKA情報：".$fuka); } // ログに残す

            // 対象の受注データ取得(会員IDの取得が目的)
            $Order = $this->app['eccube.repository.order']->find($PaydesignOrderInfo->getOrderId());
        }
        //  $objMdl->printLog("受注データ更新：受注拡張データ取得失敗"); $rcode = '9'; return $rcode; }

        // 受注データが取得できない場合は終了
        if( is_null($Order) ) {
            $objMdl->printLog("受注データ更新：受注データ取得失敗"); $rcode = '9'; return $rcode; }

        // 売上自動確定が『無効』でも、定期注文の場合は『有効』扱いとして処理を行う。
        if( $userSettings["credit_auto_confirm"] == 'false' && $fuka != $this->const['PDP_FUKA_PARAM_PERIODIC']) {
            // 売上自動確定：無効
            // 受注ステータスを決済処理中にして終了
            $Order
                ->setUpdateDate($now)
                ->setOrderStatus($this->app['eccube.repository.order_status']->find($this->app['config']['order_pending']));
        } else {
            // 売上自動確定：有効
            // 受注ステータスを新規登録にする。支払日を設定する。
            $Order
                ->setPaymentDate($now)
                ->setUpdateDate($now)
                ->setOrderStatus($this->app['eccube.repository.order_status']->find($this->app['config']['order_new']));
        }
        // 受注日が設定されていない場合は設定する（※通信不良によるデータ未設定対策）
        $order_date = $Order->getOrderDate();
        if( is_null($order_date) || empty($order_date) ) { $Order->setOrderDate($now); }
        $this->app['orm.em']->persist($Order);
        $this->app['orm.em']->flush();

        if( $userSettings["credit_auto_confirm"] == 'false' ) {
            // 売上自動確定が無効の場合の下記の定期カード情報登録はcommitOrderで対応する想定
            $objMdl->printLog("受注データ更新：受注データ取得・更新終了(売上自動確定：無効)");
            return $rcode;
        }

        // 定期カード情報が登録されてなければ、通常カード情報を定期カード情報に登録する
        if( !is_null($Order->getCustomer()) ) {
            $objClientCredit = new PAYDESIGN_PLG_DCHK_CREDIT($this->app);
            // 注文商品構成に定期商品が含まれているかどうか
            $periodic_flg = false;
            $periodic_flg = $objClientCredit->getProductPattern($Order);
            if( $periodic_flg ) {
                $customer_id = $Order->getCustomer()->getId();
                $objClientCredit->copyCardInfoToPeriodic($customer_id);
            }
        }

        $objMdl->printLog("受注データ更新：受注データ取得・更新終了");

        return $rcode;
    }
}
?>
