<?php
/*
 * Copyright(c) 2015 PAYDESIGN, Inc. All rights reserved.
 * http://www.paydesign.co.jp/
 */

namespace Plugin\PayDesign\Controller\Util;

/**
 * ペイデザイン決済プラグイン プラグイン設定情報ユーティリティクラス
 */
class PluginUtil {

    private $app;

    public function __construct(\Eccube\Application $app)
    {
        $this->app = $app;
    }

    /** サブデータを保持する変数 */
    var $subData = null;

    /** プラグイン情報 */
    var $pluginInfo = array(
        'paymentName'      => 'ペイデザイン決済サービス',
        'pluginName'       => 'ペイデザイン決済プラグイン',
        'pluginCode'       => 'PayDesign',
        'pluginVersion'    => '1.0.0',
    );

    /** 対応決済情報 */
    var $paymentInfo = array(
        'credit'      => 'クレジットカード決済',
        'convenience' => 'コンビニ決済',
    );

    /** 対応クレジットカード情報 */
    var $creditInfo = array(
        "VISA",
        "MASTER",
        "JCB",
        "AMEX",
        "DINERS",
        "CREDIT"
    );

    /** 対応コンビニ情報 */
    var $convenienceInfo = array(
         '1' => 'Loppi決済(ローソン・セイコーマート・ミニストップ)',
         '2' => 'Seven決済(セブンイレブン)',
         '3' => 'FAMIMA決済(ファミリーマート)',
        '73' => 'オンライン決済(サークルKサンクス・デイリーヤマザキ・スリーエフ)',
    );

    //  以下文字列だと長すぎたので、ヤマザキデイリーを削除
    //  '73' => 'オンライン決済(サークルKサンクス・デイリーヤマザキ・ヤマザキデイリー・スリーエフ)',

    /** 決済デフォルト情報 */
    var $dtbPaymentDefaultInfo = array(
        'creator_id'  => '2',
        'charge'      => '0',
        'charge_flg'  => '1',
        'fix_flg'     => '1',
        'create_date' => 'CURRENT_TIMESTAMP',
        'update_date' => 'CURRENT_TIMESTAMP',
        'rule_min'    => '0',
    );

    /** 定期注文申込時決済方法情報 */
    var $periodicSettleMethod = array(
        '1' => '通常決済',
        '2' => '登録のみ',
    );

    /** 申込状況 */
    var $periodicStatus = array(
        '1' => '継続中',//'現役',
        '2' => '停止',  //'休止',
        '3' => '終了',  //'解約',
        '4' => '退会',
    );

    /** EC-CUBE 旧バージョンリスト */
    /** 初期バージョン             */
    var $eccubeOldVersion     = array( '3.0.1', '3.0.2', '3.0.3', '3.0.4' );
    /** HTML記載ルール変更         */
    var $eccubeOldVersion2    = array( '3.0.5', '3.0.6', '3.0.7', '3.0.8', '3.0.9', '3.0.10' );
    /** プラグイン戻り値ルール変更 */
    var $eccubeOldVersion3    = array( '3.0.11' );
    var $eccubeOldVersionFull = array();

    /**
     * Enter description here...
     *
     * @var unknown_type
     */
    var $updateFile = array();

   /**
     * PayDesign:install()を呼んだ際に plg_paydesign_plugin のsub_dataカラムへ登録される値
     * シリアライズされて保存される.
     *
     * master_settings => 初期データなど
     * user_settings => 設定情報など、ユーザの入力によるデータ
     *
     */
    var $installSubData = array(
        // 初期データなどを保持する
        'master_settings' => array(
                                 'prefix_word' => 'PDP_',
                                 'uid_prefix_word' => 'IUID_',
                                 'credit_auto_confirm' => 'false',
                                 'credit_uid_settle' => 'false',
                                 'credit_periodic_buy' => 'false',
                                  ),
        // 設定情報など、ユーザの入力によるデータを保持する
        'user_settings' => array(),
    );

    private $pluginCode;

    /**
     * コンストラクタ
     *
     * @return void
     */
    function PluginUtil()
    {
    }

    /**
     * インスタンスを取得する
     *
     * @return PluginUtil
     */
    static function &getInstance($app)
    {
        static $paymentUtil;
        if (empty($paymentUtil)) {
            $paymentUtil = new PluginUtil($app);
        }
        $paymentUtil->init();
        return $paymentUtil;
    }

    /**
     * 初期化処理.
     */
    function init()
    {
        foreach ($this->pluginInfo as $k => $v) {
            $this->$k = $v;
        }

        // 全バージョンの格納された変数の設定
        $this->eccubeOldVersionFull = array_merge( $this->eccubeOldVersion,
                                                   $this->eccubeOldVersion2,
                                                   $this->eccubeOldVersion3 );
    }

    /**
     * 終了処理.
     */
    function destroy()
    {
    }

    /**
     * モジュール表示用名称を取得する
     *
     * @return string
     */
    function getName()
    {
        return $this->pluginName;
    }

    /**
     * 支払い方法名(決済モジュールの場合のみ)
     *
     * @return string
     */
    function getPaymentName()
    {
        return $this->paymentName;
    }

    /**
     * モジュールコードを取得する
     *
     * @param boolean $toLower trueの場合は小文字へ変換する.デフォルトはfalse.
     * @return string
     */
    function getCode($toLower = false)
    {
        $pluginCode = $this->pluginCode;
        return $pluginCode;
    }

    /**
     * モジュールバージョンを取得する
     *
     * @return string
     */
    function getVersion()
    {
        return $this->pluginVersion;
    }

    /**
     * 対応決済情報を取得する
     *
     * @return array
     */
    function getPaymentInfo()
    {
        return $this->paymentInfo;
    }

    /**
     * 対応コンビニ情報を取得する
     *
     * @return array
     */
    function getConvenienceInfo( $flg = false )
    {
        if( $flg ) {
            // 管理画面で設定されているコンビニ情報を返す
            $result = array() ;
            $userSettings = $this->getUserSettings();
            foreach($userSettings['convenience'] as $val) {
              //コンビニ選択時の表示文字列を必要最低限に調整(決済名称と括弧記号の削除処理追加)
              //->コンビニ名称だけを表示させている
              //$result[$val] = $this->convenienceInfo[$val];
                $result[$val] = preg_replace('/^.*\((.*?)\)$/u', '\1', $this->convenienceInfo[$val]);
            }
            return $result;
        } else {
            // 全てのコンビニ情報を返す
            return $this->convenienceInfo;
        }
    }

    /**
     * 対応クレジットカード情報を取得する
     *
     * @return array
     */
    function getCreditInfo()
    {
        // 全てのクレジットカード情報を返す
        return $this->creditInfo;
    }

    /**
     * クレジット決済方法名称を取得する
     *
     * @return array
     */
    function getPaymentCreditMethod()
    {
        return $this->paymentInfo['credit'];
    }

    /**
     * コンビニ決済方法名称を取得する
     *
     * @return array
     */
    function getPaymentConvenienceMethod()
    {
        return $this->paymentInfo['convenience'];
    }

    /**
     * 決済デフォルト情報を取得する
     *
     * @return array
     */
    function getDtbPaymentDefultInfo()
    {
        return $this->dtbPaymentDefaultInfo;
    }

    /**
     * 定期注文申込時決済方法情報を取得する
     *
     * @return array
     */
    function getPeriodicSettleMethod()
    {
        return $this->periodicSettleMethod;
    }

    /**
     * 申込状況を取得する
     *
     * @return array
     */
    function getPeriodicStatus()
    {
        return $this->periodicStatus;
    }

    /**
     * EC-CUBEの旧バージョン一覧を取得する
     *
     * @return array
     */
    function getEccubeOldVersion()
    {
        return $this->eccubeOldVersion;
    }

    /**
     * EC-CUBEの旧バージョン一覧を全て取得する
     *
     * @return array
     */
    function getEccubeOldVersionFull()
    {
        return $this->eccubeOldVersionFull;
    }

    /**
     * EC-CUBEのバージョンをチェック
     * 3.0.11 未満なら true、3.0.11 以降なら false を返す
     *
     * @return boolean
     */
    function checkEccubeOldVersion011()
    {
        $res = false;
        $oldVersion = array_merge( $this->eccubeOldVersion,
                                   $this->eccubeOldVersion2 );
        if( in_array(\Eccube\Common\Constant::VERSION, $oldVersion) ) { $res = true; }
        return $res;
    }

    /**
     * クレジットカードによる定期購入機能の有効・無効フラグを取得する
     *
     * @return array
     */
    function getCreditPeriodicBuy()
    {
        $subData = $this->getUserSettings();
        return $subData['credit_periodic_buy'];
    }

    /**
     * 『会員によるマイページからの次回注文日の変更を許可する』機能の有効・無効フラグを取得する
     *
     * @return array
     */
    function getChangePermissionPeriodicNextDate()
    {
        $subData = $this->getUserSettings();
        return $subData['change_permission_periodic_next_date'];
    }

    /**
     * 『会員によるマイページからの申込状況の変更を許可する』機能の有効・無効フラグを取得する
     *
     * @return array
     */
    function getChangePermissionPeriodicStatus()
    {
        $subData = $this->getUserSettings();
        return $subData['change_permission_periodic_status'];
    }

    /**
     * サブデータを取得する.
     *
     * @return mixed|null
     */
    function getSubData($key = null)
    {
        if (isset($this->subData)) {
            if (is_null($key)) {
                return $this->subData;
            } else {
                return $this->subData[$key];
            }
        }

        $pluginCode = $this->getCode(true);

        $ret = $this->app['orm.em']->getRepository('Plugin\PayDesign\Entity\PayDesignPlugin')
            ->getSubData($pluginCode);
        if (isset($ret)) {
            $this->subData = unserialize($ret);

            if (is_null($key)) {
                if(is_null($this->subData)) {
                    return $this->installSubData['master_settings'] ;
                } else {
                    return $this->subData;
                }
            } else {
                return $this->subData[$key];
            }
        }
        return null;
    }

    function getUserSettings($key = null)
    {
        $subData = $this->getSubData();
        $returnData = null;

        if (is_null($key)) {
            $returnData = isset($subData['user_settings'])
                ? $subData['user_settings']
                : $subData['master_settings'];
        } else {
            $returnData = isset($subData['user_settings'][$key])
                ? $subData['user_settings'][$key]
                : $subgData['master_settings'][$key];
        }

        return $returnData;
    }

    function registerUserSettings($data)
    {
        $this->registerSubData($data, 'user_settings');
    }

    /**
     * サブデータをDBへ登録する
     * $keyがnullの時は全データを上書きする
     *
     * @param mixed $data
     * @param string $key
     */
    function registerSubData($data, $key = null)
    {
        $subData = $this->getSubData();

        if (is_null($key)) {
            $subData = $data;
        } else {
            $subData[$key] = $data;
        }
        $subDataSer = serialize($subData);

        $pluginCode = $this->getCode(true);
        $PayDesignPlugin = $this->app['orm.em']->getRepository('Plugin\PayDesign\Entity\PayDesignPlugin')
            ->findOneBy(array('code' => $pluginCode));
        if (!is_null($PayDesignPlugin)) {
            $PayDesignPlugin->setSubData($subDataSer);
            $this->app['orm.em']->persist($PayDesignPlugin);
            $this->app['orm.em']->flush();
        }

        $this->subData = $subData;
    }

    /**
     * メール送信
     * @param  string $subject
     * @param  string $to_mailaddr
     * @param  mixed  $body
     * @return none
     */
    public function sendMail( $subject, $to_mailaddr, $body, $bcc_flg = true ) {

        // Start
        $this->printLog("メール送信[件名]：".$subject." 送信開始");
        $this->printLog("送信先アドレス：".$to_mailaddr);

        // システム基本情報の取得
        $BaseInfo = $this->app['eccube.repository.base_info']->get();

        // メール送信データ作成
        if( !is_null($BaseInfo) ) {
            $message = \Swift_Message::newInstance()
                ->setSubject($subject)
                ->setFrom(array($BaseInfo->getEmail03() => $BaseInfo->getShopName()))
                ->setTo(array($to_mailaddr))
                ->setReturnPath($BaseInfo->getEmail04())
                ->setBody($body);

            // Bccは指定がある場合のみ(デフォルトは送信する)
            if( $bcc_flg ) {
                $message
                    ->setBcc(array($BaseInfo->getEmail01() => $BaseInfo->getShopName()));
                $this->printLog("Bccアドレス：".$BaseInfo->getEmail01());
            }

            // メール送信
            $this->app->mail($message);

            // End
            $this->printLog("メール送信[件名]：".$subject." 送信完了");
        } else {
            // End
            $this->printLog("基本情報取得失敗：メール送信エラー");
        }
    }

    /**
     * ログを出力.
     *
     * @param string $msg
     * @param mixed $data
     */
    function printLog($msg, $date = null)
    {
        $path = $this->app['config']['root_dir'] . "/app/log/" . $this->getCode(true) . '_' . date('Ymd') . '.log';

        $text = '';
        if (is_array($msg)) {
            $text = print_r($msg, true);
        } else {
            $text = $msg;
        }
        CommonUtil::printLog($this->app, $text, $path);
    }

    /**
     * インストール処理
     *
     * @param boolean $force true時、上書き登録を行う
     */
    function install($force = false)
    {
        $subData = $this->getSubData();
        if (is_null($subData) || $force) {
            $this->registerSubdata(
                $this->installSubData['master_settings'],
                'master_settings'
            );
        }
    }

    /**
     * When uninstalling update del_flg of paydesign payment method and update del_flg of page layout
     */
    public function uninstallPayment()
    {
        $arrPayment = $this->app['orm.em']->getRepository('\Plugin\PayDesign\Entity\PayDesignPaymentMethod')->getPaymentList();

        foreach ($arrPayment as $payment_id) {
            // Update del_flg of paydesign payment method
            $Payment = $this->app['orm.em']->getRepository('\Eccube\Entity\Payment')->findOneBy(array('id' => $payment_id));
            $Payment->setUpdateDate(new \DateTime());
            $Payment->setDelFlg(1);
            // Update del_flg of page layout
            $DeviceType = $this->app['eccube.repository.master.device_type']->find(10);
            $PageLayout = $this->app['eccube.repository.page_layout']->getByUrl($DeviceType, 'shopping/paydesign');
            $PageLayout->setEditFlg('1');

            $this->app['orm.em']->persist($Payment);
            $this->app['orm.em']->persist($PageLayout);
            $this->app['orm.em']->flush();
        }
    }
}
?>
