<?php
/*
 * Copyright(c) 2015 PAYDESIGN, Inc. All rights reserved.
 * http://www.paydesign.co.jp/
 */

namespace Plugin\PayDesign\Form\Type;

use Plugin\PayDesign\Controller\Util\CommonUtil;
use Plugin\PayDesign\Controller\Util\PluginUtil;
use \Symfony\Component\Form\AbstractType;
use \Symfony\Component\Form\Extension\Core\Type;
use \Symfony\Component\Form\FormBuilderInterface;
use \Symfony\Component\Validator\Constraints as Assert;

class AdminSearchPeriodicType extends AbstractType
{
    protected $app;
    protected $config;

    public function __construct($app)
    {
        $this->app = $app;
        $this->config = $app['config'];
    }

    /**
     * Build config type form
     *
     * @param FormBuilderInterface $builder
     * @param array $options
     * @return type
     */
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $config = $this->config;

        // 申込状況一覧を取得
        $objPluginUtil =& PluginUtil::getInstance($this->app);
        $objPluginUtil->install();
        $arrPeriodicStatus = $objPluginUtil->getPeriodicStatus();

        $builder
            // 定期ID・購入者名
            ->add('multi', 'text', array(
                'label' => '定期ID・購入者名',
                'required' => false,
                'constraints' => array(
                    new Assert\Length(array('max' => $config['stext_len'])),
                ),
            ))
            ->add('status', 'choice', array(
                'label' => '申込状況',
                'choices' => $arrPeriodicStatus,
                'multiple'=> false,
                'expanded' => false,
                'required' => false,
                'empty_value' => '-',
                'empty_data' => '',
            ))
            ->add('name', 'text', array(
                'required' => false,
            ))
            ->add('kana', 'text', array(
                'required' => false,
            ))
            ->add('email', 'email', array(
                'required' => false,
            ))
            ->add('periodic_next_date_start', 'date', array(
                'label' => '次回注文日(FROM)',
                'required' => false,
                'input' => 'datetime',
                'widget' => 'single_text',
                'format' => 'yyyy-MM-dd',
                'empty_value' => array('year' => '----', 'month' => '--', 'day' => '--'),
            ))
            ->add('periodic_next_date_end', 'date', array(
                'label' => '次回注文日(TO)',
                'required' => false,
                'input' => 'datetime',
                'widget' => 'single_text',
                'format' => 'yyyy-MM-dd',
                'empty_value' => array('year' => '----', 'month' => '--', 'day' => '--'),
            ))
            /*
            ->add('tel', 'tel', array(
                'required' => false,
            ))
            */
            ->add('tel', 'text', array(
                'required' => false,
                'constraints' => array(
                    new Assert\Regex(array(
                        'pattern' => "/^[\d-]+$/u",
                        'message' => 'form.type.admin.nottelstyle',
                    )),
                ),
            ))
            ->add('payment', 'payment', array(
                'label' => '支払方法',
                'required' => false,
                'expanded' => true,
                'multiple' => true,
            ))
            ->add('product_price_start', 'integer', array(
                'label' => '商品金額(下限)',
                'required' => false,
            ))
            ->add('product_price_end', 'integer', array(
                'label' => '商品金額(上限)',
                'required' => false,
            ))
            ->add('buy_product_name', 'text', array(
                'label' => '購入商品名',
                'required' => false,
            ))
            ->addEventSubscriber(new \Eccube\Event\FormEventSubscriber());
    }

    /**
     * {@inheritdoc}
     */
    public function getName()
    {
        return 'admin_search_periodic';
    }
}
?>
