<?php
/*
 * Copyright(c) 2015 PAYDESIGN, Inc. All rights reserved.
 * http://www.paydesign.co.jp/
 */

namespace Plugin\PayDesign\Service\client;

require_once(__DIR__ . "/../../vendor/autoload.php");

use Eccube\Common\Constant;
use Guzzle\Service\Client;
use Plugin\PayDesign\Controller\Util\PluginUtil;
use Plugin\PayDesign\Controller\Util\CommonUtil;

/**
 * 決済モジュール 決済処理 基底クラス
 */
class PAYDESIGN_PLG_DCHK
{

    public $error = array();
    public $results = null;
    protected $app;
    protected $const;

    /**
     * コンストラクタ
     *
     * @return void
     */
    function __construct(\Eccube\Application $app)
    {
        $this->app = $app;
        $this->const = $app['config']['PayDesign']['const'];
    }

    /*
     * http URL作成プログラム
     */
    function makeLinkUrl($url, $sendData) {

        // GETパラメータをSJISエンコード
        $listData = array();
        foreach ($sendData as $key => $value) {
            $listData[$key] = mb_convert_encoding($value, 'SJIS-win', 'UTF-8');
        }
        return $url."?".http_build_query($listData);
    }

    /*
     * http 送受信プログラム
     * $url に $sendData のデータを送信し、レスポンスを返す
     */
    function sendRequest($url, $sendData) {

        $listData = array();
        foreach ($sendData as $key => $value) {
            $listData[$key] = mb_convert_encoding($value, 'SJIS-win', 'UTF-8');
        }
        $client = new Client();
        $request = $client->post($url, array(), $listData);
        $response = $request->send();

        $r_code = $response->getStatusCode();
        switch ($r_code) {
            case 200:
                break;
            case 404:
                $msg = 'レスポンスエラー:RCODE:' . $r_code;
                $this->setError($msg);
                return false;
                break;
            case 500:
            default:
                $msg = '決済サーバーエラー:RCODE:' . $r_code;
                $this->setError($msg);
                return false;
                break;
        }

        // レスポンスデータを取得
        $response_body = $response->getBody(true);
        if (is_null($response_body)) {
            $msg = 'レスポンスデータエラー: レスポンスがありません。';
            $this->setError($msg);
            return false;
        }

        // レスポンスデータを返す（文字コード変換と各行を配列に設定）
        $response_body = explode("\r\n", mb_convert_encoding($response_body,"UTF-8","SJIS"));

        // 正常時はレスポンスデータをログに出力
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $objMdl->printLog("送信先URL：".$url);
        $objMdl->printLog("取得したレスポンス：");
        $objMdl->printLog($response_body);

        return $response_body;
    }

    function setResults($results) {
        $this->results = $results;
    }

    function getResults() {
        if (is_null($this->results[0]) && !is_null($this->results)) {
            return $this->results;
        }
        return $this->results[0];
    }

    function setError($msg) {
        $this->error[] = $msg;
    }

    function getError() {
        return $this->error;
    }

    /*
     * 注文完了処理
     */
    function changeOrderData($Order, $periodic_flg = false) {

        $formData = $this->app['session']->get('paydesign_formData');
        $this->app['session']->remove('paydesign_formData');

        $em = $this->app['orm.em'];
        $em->getConnection()->beginTransaction();
        $Order->setOrderDate(new \DateTime());
        $Order->setMessage($formData['message']);

        // 設定情報を取得
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // クレジット、コンビニ決済拡張情報を取得
        $payment_id = $Order->getPayment()->getId();
        $PayDesignPaymentMethod = $this->app['eccube.plugin.paydesign.repository.paydesign_payment_method']->find($payment_id);
        // クレジット決済で売上自動確定が無効の場合のみチェックを行う
        if( $PayDesignPaymentMethod->getMemo03() == "credit" && $userSettings["credit_auto_confirm"] == 'false' ) {
            // 決済要求結果取得日時を取得
            $PaydesignOrderInfo =
                $this->app['eccube.plugin.paydesign.repository.paydesign_order_info']->findOneBy( array( 'order_id' => $Order->getId(), 'order_kind' => $this->app['config']['PayDesign']['const']['PDP_ORDER_KIND_NORMAL']));
            if( !is_null($PaydesignOrderInfo) ) {
                $result_receive_date = $PaydesignOrderInfo->getResultReceiveDate();
                // 売上自動確定が無効で決済要求結果を取得済みで、受注ステータスが【新規登録】になっている
                // 場合は【決済処理中】に戻す。
                // 定期注文の場合($periodic_flg = true)は処理不要
                if( !is_null($result_receive_date) && isset($result_receive_date) && ( $Order->getOrderStatus()->getId() == $this->app['config']['order_new'] ) && !$periodic_flg ) { $Order->setOrderStatus($this->app['eccube.repository.order_status']->find($this->app['config']['order_pending'])); }
            }
        }

        // コンビニ決済の場合は受注ステータスを【入金待ち】にする
        if( $PayDesignPaymentMethod->getMemo03() == "convenience" ) {
            $Order->setOrderStatus($this->app['eccube.repository.order_status']->find($this->app['config']['order_pay_wait']));
        }

        // 旧バージョン(3.0.4以下)限定処理
        $oldVersion = $objMdl->getEccubeOldVersion();
        if( in_array(Constant::VERSION, $oldVersion) ) {
            // お届け先情報を更新
            $shippings = $Order->getShippings();
            foreach ($shippings as $shipping) {
                $shipping->setShippingDeliveryName($formData['delivery']->getName());
                if (!empty($formData['deliveryTime'])) {
                    $shipping->setShippingDeliveryTime($formData['deliveryTime']->getDeliveryTime());
                }
                if (!empty($formData['deliveryDate'])) {
                    $shipping->setShippingDeliveryDate(new \DateTime($formData['deliveryDate']));
                }
                $shipping->setShippingDeliveryFee($shipping->getDeliveryFee()->getFee());
            }
        }

        $orderService = $this->app['eccube.service.order'];
        $orderService->setStockUpdate($em, $Order);
        if ($this->isGranted($this->app)) {
            // 会員の場合、購入金額を更新
            $orderService->setCustomerUpdate($em, $Order, $this->app->user());
        }
        $em->getConnection()->commit();
        $em->flush();

      // ここでEntityManagerはCloseしてはいけない（バグ防止の注意書きとして残す）
      //$em->close();
    }

    protected function isGranted($app) {

        if ($this->app['security']->isGranted('ROLE_USER')) {
            return true;
        }
        return false;
    }

    /*
     * 【送信用】決済共通パラメータ設定
     */
    function getBasicSendData($sid, $payment_method, $pass_flg = false, $fuka_param = NULL) {

        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        switch($payment_method) {
            case 'credit':
                $ip_code = $userSettings['credit_shop_code'];
                $pass    = $userSettings['credit_shop_passwd']; 
                break ;
            case 'convenience':
                $ip_code = $userSettings['conv_shop_code'];
                $pass    = $userSettings['conv_shop_passwd']; 
                break ;
        }

        // パラメータに応じてFUKA情報を設定する
        $fuka = "";
        switch($fuka_param) {
            case 1: // 定期注文である事を示す
                $fuka = $this->const['PDP_FUKA_PARAM_PERIODIC'];
                break;
            default:
                break;
        }

        $pass_msg = "(送信しない)";
        if($pass_flg) { $pass_msg = "(送信する)"; }

        // ログ
        $msg  = "決済 共通パラメータ：\n" ;
        $msg .= "　1.IP   ... ".$ip_code."\n";
        $msg .= "　2.SID  ... ".$sid."\n";
        $msg .= "　3.PASS ... ".$pass.$pass_msg."\n"; 
        $msg .= "　4.FUKA ... ".$fuka."\n";
        $objMdl->printLog($msg);

        // データ設定
        $arrRet = array(
            'IP'   => $ip_code,
            'SID'  => $sid,
            'FUKA' => $fuka, 
        );
        if( $pass_flg ) { $arrRet['PASS'] = $pass; }

        return $arrRet;
    }

    /*
     * 【送信用】決済注文パラメータ設定
     */
    function getOrderSendData($Order, $uid_flg = false) {

        // 汎用ユーティリティ（オブジェクト）の作成
        $objCommonUtil = new CommonUtil();

        // データを取得
        if( !$uid_flg ) {
            $name01 = $objCommonUtil->toRegularString($Order->getName01(), 10);
            $name02 = $objCommonUtil->toRegularString($Order->getName02(), 10);
            $kana01 = $objCommonUtil->toRegularString($Order->getKana01(), 10);
            $kana02 = $objCommonUtil->toRegularString($Order->getKana02(), 10);
            $tel    = $Order->getTel01().$Order->getTel02().$Order->getTel03();
            $email  = $Order->getEmail();
            $zip01  = $Order->getZip01();
            $zip02  = $Order->getZip02();
            $addr01 = $objCommonUtil->toRegularString($Order->getPref().$Order->getAddr01(), 25);
            $addr02 = $objCommonUtil->toRegularString($Order->getAddr02(), 25);
        }
        $k1     = $Order->getPaymentTotal();

        // 商品名は最後の商品名が記述されるようにする
        $OrderDetails = $Order->getOrderDetails();
        foreach ($OrderDetails as $OrderDetail) {
            $n1 = $objCommonUtil->toRegularProductString($OrderDetail->getProductName(), 19);
        }

        // ログ
        $objMdl =& PluginUtil::getInstance($this->app);
        $msg  = "決済 注文パラメータ：\n" ;
        if( !$uid_flg ) {
            $msg .= "　 1.NAME1  ... ".$name01."\n";
            $msg .= "　 2.NAME2  ... ".$name02."\n";
            $msg .= "　 3.KANA1  ... ".$kana01."\n";
            $msg .= "　 4.KANA2  ... ".$kana02."\n";
            $msg .= "　 5.YUBIN1 ... ".$zip01."\n";
            $msg .= "　 6.YUBIN2 ... ".$zip02."\n";
            $msg .= "　 7.TEL    ... ".$tel."\n";
            $msg .= "　 8.ADR1   ... ".$addr01."\n";
            $msg .= "　 9.ADR2   ... ".$addr02."\n";
            $msg .= "　10.MAIL   ... ".$email."\n";
            $msg .= "　11.N1     ... ".$n1."\n";
            $msg .= "　12.K1     ... ".$k1;
        } else {
            $msg .= "　 1.N1     ... ".$n1."\n";
            $msg .= "　 2.K1     ... ".$k1;
        }
        $objMdl->printLog($msg);

        // データを設定
        if( !$uid_flg ) {
            $arrRet = array(
                'NAME1'  => $name01,
                'NAME2'  => $name02,
                'KANA1'  => $kana01,
                'KANA2'  => $kana02,
                'YUBIN1' => $zip01,
                'YUBIN2' => $zip02,
                'TEL'    => $tel,
                'ADR1'   => $addr01,
                'ADR2'   => $addr02,
                'MAIL'   => $email,
                'N1'     => $n1,
                'K1'     => $k1,
            );
        } else {
            $arrRet = array(
                'N1'     => $n1,
                'K1'     => $k1,
            );
        }

        return $arrRet ;
    }

    /*
     * 注文商品構成に定期商品が含まれているかどうか
     */
    function checkPeriodicProducts($Order)
    {
        // 初期化
        $periodic_flg = false ;
        // 商品詳細情報一覧の取得
        $orderDetails = $Order->getOrderDetails();

        foreach ($orderDetails as $orderDetail) {
            // 対象注文内商品の商品定期情報を取得
            $product_class_id = $orderDetail->getProductClass()->getId();
            $ProductPeriodicInfo = $this->app['eccube.plugin.paydesign.repository.paydesign_product_periodic_info']->find($product_class_id);
            // 定期商品設定されているかをチェック（定期商品の場合は設定して処理終了）
            if( !is_null($ProductPeriodicInfo) ) {
                if( $ProductPeriodicInfo->getPeriodicFlg() > 0 ) { $periodic_flg = true; break; }
            }
        }

        return $periodic_flg ;
    }

    /*
     * 注文商品構成から購入パターンを取得する
     * 
     */
    function getProductPattern($Order)
    {
        $productPattern = 0;
        $normal_flg = false;
        $regist_only_flg = false;

        if( !is_null($Order) ) {
           // 注文詳細情報を取得
           $OrderDetails = $Order->getOrderDetails() ;
           foreach( $OrderDetails as $OrderDetail ) {
               $ProductClass = $OrderDetail->getProductClass();
               if( !is_null($ProductClass) ) {
                   // 注文商品の定期情報を取得
                   $ProductPeriodicInfo = $this->app['eccube.plugin.paydesign.repository.paydesign_product_periodic_info']->find($ProductClass->getId());
                   if( !is_null($ProductPeriodicInfo) ) {
                       // 定期商品設定されているかをチェック
                       if( $ProductPeriodicInfo->getPeriodicFlg() > 0 ) {
                           // 申込時決済方法を取得
                           $periodic_settle_method = $ProductPeriodicInfo->getPeriodicSettleMethod();
                           if( $periodic_settle_method == 1 ) {
                               // 通常決済
                               $normal_flg = true;
                           } else {
                               // 登録のみ
                               $regist_only_flg = true;
                           }
                       } else {
                           // 定期商品設定されていない場合は、通常決済と判断する
                           $normal_flg = true;
                       }
                   } else {
                       // 注文商品の定期情報が無い場合は、通常決済と判断する
                       $normal_flg = true;
                   }
               }
           }
        }

        // 通常商品も登録のみ商品も含まれている場合
        if( $normal_flg && $regist_only_flg ) { $productPattern = 1; }

        // 登録のみ商品だけの場合
        if( !$normal_flg && $regist_only_flg ) { $productPattern = 2; }


        return $productPattern;
    }

    /*
     * 2017.1 plg_paydesign_order_info の初期データ作成
     */
    function initPaydesignOrderInfo($Order, $kind, $uid_prefix = '',
                                    $credit_uid_card_select_flg = 0, $check_count = 0 ) {

        $PaydesignOrderInfo = NULL;

        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        $objMdl->printLog("拡張受注情報作成開始");

        if( !is_null($Order) ) {
            // 拡張受注情報を存在していなければ作成
            $PaydesignOrderInfo = $this->app['eccube.plugin.paydesign.repository.paydesign_order_info']->findOneBy( array( 'order_id' => $Order->getId(), 'order_kind' => $kind) );
            if( is_null($PaydesignOrderInfo) ) {
                $PaydesignOrderInfo = new \Plugin\PayDesign\Entity\PayDesignOrderInfo() ; }

            // データの設定
            $PaydesignOrderInfo
                ->setOrderId($Order->getId())
                ->setSid($uid_prefix.$userSettings['prefix_word'].$Order->getId())
                ->setOrderKind($kind)
                ->setCreditUidCardSelectFlg($credit_uid_card_select_flg)
                ->setCheckCount($check_count);

            // 登録・更新処理実行
            $this->app['orm.em']->persist($PaydesignOrderInfo);
            $this->app['orm.em']->flush();
        } else {
            $objMdl->printLog("拡張受注情報作成 受注情報未取得エラー");
        }

        $objMdl->printLog("拡張受注情報作成終了");
        return $PaydesignOrderInfo;
    }
}
?>
